/*
 * vim: ts=4 nobackup
 */


#include <stdint.h>
#include <stdlib.h>
#include "Timer.H"
#include "KeyMatrixManager.H"


using namespace avelino;
using namespace std;


KeyMatrixManager::KeyMatrixManager() {
	this->listener = NULL;
}


void KeyMatrixManager::init(KeyMatrixListener &l) {
	this->listener = &l;
	this->timerHandler = Timer::alloc();
	Timer::counter[this->timerHandler] = 0;
	this->configurePorts();
	this->status = STATUS_SELECT_ROW;
	this->keyRow = 0;
}


void KeyMatrixManager::run() {
	if (Timer::counter[this->timerHandler] > 0)
		return;
	if (this->status == STATUS_SELECT_ROW) {
		this->enableRow(this->keyRow);
		this->status = STATUS_READ_COLUMN;
	}
	else if (this->status == STATUS_READ_COLUMN) {
		uint8_t numRows = this->getNumRows();
		uint8_t numColumns = this->getNumColumns();
		bool found = false;
		for (uint8_t i = 0; (i < numColumns) && !found; i++) {
			if (this->getColumn(i)) {
				found = true;
				this->keyColumn = i;
			}
		}
		if (found) {
			this->status = STATUS_KEY_PRESSED;
			this->listener->keyPressed(this->keyRow, this->keyColumn);
			this->waitCounter = 5;
		}
		else {
			this->status = STATUS_SELECT_ROW;
			this->keyRow++;
			if (this->keyRow == numRows)
				this->keyRow = 0;
		}
	}
	else if (this->status == STATUS_KEY_PRESSED) {
		if (this->waitCounter == 0) {
			this->status = STATUS_WAIT_KEY_RELEASE;
		}
		else
			this->waitCounter--;
	}
	else if (this->status == STATUS_WAIT_KEY_RELEASE) {
		if (!this->getColumn(this->keyColumn)) {
			this->listener->keyReleased(this->keyRow, this->keyColumn);
			this->status = STATUS_SELECT_ROW;
			this->keyRow = 0;
		}
	}
	Timer::counter[this->timerHandler] = 10;
}


const uint8_t KeyMatrixManager::STATUS_SELECT_ROW = 0;
const uint8_t KeyMatrixManager::STATUS_READ_COLUMN = 1;
const uint8_t KeyMatrixManager::STATUS_KEY_PRESSED = 2;
const uint8_t KeyMatrixManager::STATUS_WAIT_KEY_RELEASE = 3;


